import { Http, Loader, Mask } from '@zeedhi/core';
import { InstanceService } from '@/services/InstanceService';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { DialogService, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import Messages from '@/services/Messages';
import { Utilities } from '@/services/Utilities';

Mask.addPattern('9', /[0-9]/);

export class DMTabSolicitationEditController {
	public demandManagementController = Loader.getInstance('DemandManagementController');

	public defaultTimeoutTimer: any;

	private modalChangeResponsible!: Modal;

	private modalSoliOriginEditPesq!: Modal;

	private modalAddItemTaskSoli!: Modal;

	private modalSoliOriginAddItemTask!: Modal;

	public CDFASETRAM = '';

	public CALL_RELATIONSHIP = '';

	public CDCLIENTE = '';

	public fieldSetOriSoli = '';

	public fieldSetOriSoliAddTaskItem = '';

	private SGSISTEMA = '';

	public isCallTypeS() {
		return this.demandManagementController.callType === 'S';
	}

	public getFormEdit() {
		return InstanceService.getInstanceZdForm('formDemandManagementS');
	}

	/** Eventos de fields */

	public async onChangeClientEditCallModal(params: any) { // CLIENT INPUT EDIT
		const useLicenseAddS = InstanceService.getInstanceZdSelect('useLicenseEdit');
		if (params.value) {
			this.CDCLIENTE = params.component.value;
			try {
				await useLicenseAddS.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}
	}

	public async onChangeProductEditCallModal(params: any) {
		if (params.value) {
			const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');

			this.SGSISTEMA = params.component.selectedData.SGSISTEMA;
			try {
				await productTypeInstance.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}
	}

	public async onFocusProductEditCall() {
		const productInstance = InstanceService.getInstanceZdSelect('productEdit');
		delete productInstance.datasource.filter.CALL_RELATIONSHIP;
		delete productInstance.datasource.filter.CDPRODCOMP;

		await productInstance.datasource.get();
	}

	public async rowClickModalSoliOriginEditPesq(params: any) {
		const soliOriginEdit = InstanceService.getInstanceTextInput(this.fieldSetOriSoli);
		soliOriginEdit.value = params.row.NRSOLISERV;
		this.closeModalSoliOriginEditPesq();
	}

	public async changeSoliOriginEdit(params: any) {
		if (params.component.value) {
			params.component.value = `0000000000${params.component.value}`.slice(-10);
			try {
				const response = await Http.get(`findAllSoliserv?page=1&limit=25&search=${params.component.value}&search_in%5B0%5D=NRSOLISERV`);
				if (!response.data.data[0]) {
					DialogServiceCustom.showError(Messages.ERROR_SOLI_ORI);
				}
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}
	}

	public async onChangeProductTypeEditCallModal(params: any) { // PRODUCT TYPE EDIT
		// clearTimeout(this.defaultTimeoutTimer);
		// this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				versionSInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
				versionSInstance.datasource.filter.PRODUCT_CONCAT = `${params.component.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;

				try {
					const version = await versionSInstance.datasource.get();
					if (version.length === 1) {
						this.demandManagementController.versionNOTSelected = false;
						await versionSInstance.setValue(version[0].ID, false);
						releaseInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
						releaseInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
						const release = await releaseInstance.datasource.get();
						if (release.length === 1) {
							this.demandManagementController.releaseNOTSelected = false;
							releaseInstance.value = release[0].ID;
							patchInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
							patchInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
							patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
							await patchInstance.datasource.get();
						}
					}
					this.demandManagementController.productTypeNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.productTypeNOTSelected = true;
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
					versionSInstance.value = '';
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.demandManagementController.productTypeNOTSelected = true;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				versionSInstance.value = '';
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		// }, 350);
	}

	public onChangeVersionSEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				releaseInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
				try {
					const release = await releaseInstance.datasource.get();
					if (release.length === 1) {
						releaseInstance.value = release[0].ID;
						this.demandManagementController.releaseNOTSelected = false;
						patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
						patchInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
						patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
						await patchInstance.datasource.get();
					}
					this.demandManagementController.versionNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.versionNOTSelected = true;
					this.demandManagementController.releaseNOTSelected = true;
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				debugger;
				this.demandManagementController.versionNOTSelected = true;
				this.demandManagementController.releaseNOTSelected = true;
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onChangeReleaseEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');

				patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.value;
				patchInstance.datasource.filter.PRODUCT_RELEASE = params.component.value;

				try {
					await patchInstance.datasource.get();
					this.demandManagementController.releaseNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.releaseNOTSelected = true;
					patchInstance.value = '';
				}
			} else {
				this.demandManagementController.releaseNOTSelected = true;
				patchInstance.value = '';
			}
		}, 350);
	}

	public resetValidationsEDIT() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		callFormInstance.resetValidation();
	}

	public resetInputtedValuesByClientEditField() {
		const callFormInstance = this.getFormEdit();
		callFormInstance.value.categoryEdit = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
	}

	public resetInputtedValuesByProductEditField() {
		const callFormInstance = this.getFormEdit();
		callFormInstance.value.categoryEdit = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
	}

	public async changeResponsibleSoli(param: any) {
		this.modalChangeResponsible = await ModalService.createFromJson('demandManagement/modal/changeResponsibleSoliDemandManagementModal', true);
		await this.modalChangeResponsible.show();
		const forwardToInstance = InstanceService.getInstanceZdSelect('forwardToChangeResponsibleSoliModal');
		if (param.component.events && param.component.events.ROW) {
			const row = param.component.events.ROW();
			this.CDCLIENTE = row.CDCLIENTE;
			this.CALL_RELATIONSHIP = row.CALL_RELATIONSHIP;
			param.component.events.closeModalSoliActions();
		} else {
			const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
			this.CDCLIENTE = callFormInstance.value.customerEdit;
			this.CALL_RELATIONSHIP = callFormInstance.value.teknisaIdEdit;
		}
		await forwardToInstance.datasource.get();
	}

	public closeModalChangeResponsibleSoli() {
		this.modalChangeResponsible.hide();
		this.modalChangeResponsible.destroy();
	}

	public closeModalSoliOriginEditPesq() {
		this.modalSoliOriginEditPesq.hide();
		this.modalSoliOriginEditPesq.destroy();
	}

	public saveChangeResponsibleSoli() {
		const forwardToInstance = InstanceService.getInstanceZdForm('formChangeResponsibleSoliModal');
		const checkValidation = forwardToInstance.validate();
		if (checkValidation) {
			const message = Messages.CHANGE_RESP;
			const btnYes = [{
				name: Utilities.getNameDynamic('yes'),
				component: 'ZdButton',
				label: 'Sim',
				outline: false,
				events: {
					click: (() => {
						DialogService.hide();
						this.saveChangeResponsible();
					}),
				},
			}];
			DialogServiceCustom.confirmMessage(message, btnYes);
		}
	}

	public async clickSoliOriginEditPesq() {
		this.modalSoliOriginEditPesq = await ModalService.createFromJson('demandManagement/modal/modalSoliOriginEditPesq', true);
		this.fieldSetOriSoli = 'soliOriginAdd';
		await this.modalSoliOriginEditPesq.show();
	}

	public async clickSoliOriginEditPesqEdit() {
		this.modalSoliOriginEditPesq = await ModalService.createFromJson('demandManagement/modal/modalSoliOriginEditPesq', true);
		this.fieldSetOriSoli = 'soliOriginEdit';
		await this.modalSoliOriginEditPesq.show();
	}

	public async clickSoliOriginEditPesqAdd() {
		this.modalSoliOriginEditPesq = await ModalService.createFromJson('demandManagement/modal/modalSoliOriginEditPesq', true);
		this.fieldSetOriSoli = 'soliOriginAddS';
		await this.modalSoliOriginEditPesq.show();
	}

	public async saveChangeResponsible() {
		const forwardToInstance = InstanceService.getInstanceZdForm('formChangeResponsibleSoliModal');
		const checkValidation = forwardToInstance.validate();
		if (checkValidation) {
			const row = {
				CALL_RELATIONSHIP: this.CALL_RELATIONSHIP,
				CDOPERADOR: forwardToInstance.value.forwardToChangeResponsibleSoliModal,
				COMMENT: forwardToInstance.value.observationChangeResponsibleSoliModal,
				requestType: 'DataSet',
			};
			LoadingService.show();
			await Http.post('/sendChangeRespon', row);
			const selectChangeResponsible = InstanceService.getInstanceZdSelect('forwardToChangeResponsibleSoliModal');
			const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
			callFormInstance.value.responsibleNameByEdit = selectChangeResponsible.selectedData.NMOPERADOR;
			this.closeModalChangeResponsibleSoli();
			LoadingService.hide();
		}
	}

	public async saveEditCall() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		const checkValidation = callFormInstance.validate();
		if (checkValidation) {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
			LoadingService.show();
			const productInstance = InstanceService.getInstanceZdSelect('productEdit');
			const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
			const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
			try {
				await Http.post('/productCallSave', {
					CALL_RELATIONSHIP: this.demandManagementController.callRelationship,
					ID: this.demandManagementController.callID,
					REFERENCE: callFormInstance.value.referenceEdit,
					PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
					MENU_ITEM: '',
					MENU_VERSION: '',
					NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
					CALL_LABELS: callFormInstance.value.labelsEdit,
					DESCRIPTION: callFormInstance.value.descriptionEdit,
					REPORTED_BY: callFormInstance.value.reportedByEdit,
					CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
					PLATAFORMA: 'DELPHI',
					PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
					SGSISTEMA: productInstance.selectedData.SGSISTEMA,
					TYPE: 'S',
					PRODUCT: versionSInstance.selectedData.CDPRODCOMP,
					PRODUCT_VERSION: callFormInstance.value.versionEditS,
					PRODUCT_RELEASE: callFormInstance.value.releaseEdit,
					PRODUCT_TYPE: callFormInstance.value.productTypeEdit,
					CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
					CLASSIFICATION: callFormInstance.value.categoryEdit,
					PRODUCT_PATCH: callFormInstance.value.patchEdit,
					USE_LICENSE: callFormInstance.value.useLicenseEdit,
					APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					NRSOLIORIG: callFormInstance.value.soliOriginEdit,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await callGridInstance.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public async additemTaskSoli() {
		LoadingService.show();
		this.modalAddItemTaskSoli = await ModalService.createFromJson('demandManagement/modal/addItemTaskSoliModal', true);
        const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		await this.modalAddItemTaskSoli.show();
		InstanceService.getInstanceTextInput('soliOrigin-formAddItemTaskSoliModal').value = callFormInstance.value.soliOriginEdit;
		LoadingService.hide();
	}

	public closeModalAddItemTaskSoli() {
		this.modalAddItemTaskSoli.hide();
		this.modalAddItemTaskSoli.destroy();
	}

	public async rowClickModalSoliOriginPesqAddItemTask(params: any) {
		const soliOriginFormAddItemTaskSoliModal = InstanceService.getInstanceTextInput(this.fieldSetOriSoliAddTaskItem);
		soliOriginFormAddItemTaskSoliModal.value = params.row.NRSOLISERV;
		this.closeModalSoliOriginAddItemTask();
	}

	public async changeSoliOriginAddItemTaskModal(params: any) {
		if (params.component.value) {
			params.component.value = `0000000000${params.component.value}`.slice(-10);
			try {
				const response = await Http.get(`findAllSoliserv?page=1&limit=25&search=${params.component.value}&search_in%5B0%5D=NRSOLISERV`);
				if (!response.data.data[0]) {
					DialogServiceCustom.showError(Messages.ERROR_SOLI_ORI);
				}
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		}
	}

	public async clickSoliOriginAddItemTaskModal() {
		this.modalSoliOriginAddItemTask = await ModalService.createFromJson('demandManagement/modal/modalSoliOriginAddItemTask', true);
		this.fieldSetOriSoliAddTaskItem = 'soliOrigin-formAddItemTaskSoliModal';
		await this.modalSoliOriginAddItemTask.show();
	}

	public closeModalSoliOriginAddItemTask() {
		this.modalSoliOriginAddItemTask.hide();
		this.modalSoliOriginAddItemTask.destroy();
	}

	public async saveItemTaskSoli() {
		const gridItemTaskS = InstanceService.getInstanceTekGrid('gridItemTaskS');
        const gridTaskS = InstanceService.getInstanceTekGrid('gridTaskS');
        const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		LoadingService.show();
		try {
			await Http.post('/saveItemTaskSoli', {
				NRSOLISERV: callFormInstance.value.teknisaIdEdit,
				NRSOLIORIG: InstanceService.getInstanceTextInput('soliOrigin-formAddItemTaskSoliModal').value,
                NRTAREFA: gridItemTaskS.datasource.filter.NRTAREFA,
                DSREFETAREFA: gridTaskS.datasource.currentRow.DSREFETAREFA,
			});
			AlertServiceCustom.defaultSaveSuccessAlert();
			await gridItemTaskS.datasource.get();
			this.closeModalAddItemTaskSoli();
		} catch (e) {
			LoadingService.hide();
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}
}
